"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
class BufferReader {
    constructor(buf) {
        this.buf = buf;
        this.pos = 0;
    }
    nextInt16LE() {
        const num = this.buf.readInt16LE(this.pos);
        this.pos += 2;
        return num;
    }
    nextBuffer(length) {
        const buf = this.buf.slice(this.pos, this.pos + length);
        this.pos += length;
        return buf;
    }
    nextString(length) {
        return this.nextBuffer(length).toString('utf16le');
    }
    hasNext() {
        return this.pos < this.buf.length;
    }
}
class ScelParser {
    constructor(buf) {
        this.buf = buf;
    }
    parseInfo() {
        const extract = (from, to) => this.buf.slice(from, to).toString('utf16le').split('\x00', 1)[0];
        return {
            name: extract(0x130, 0x338),
            type: extract(0x338, 0x554),
            description: extract(0x540, 0xd40),
            example: extract(0xd40, 0x1540),
        };
    }
    parsePinyinTable() {
        const reader = new BufferReader(this.buf.slice(0x1540, 0x2628));
        if (!reader.nextBuffer(4).compare(Buffer.from('\x9d\x01\x00\x00'))) {
            throw new Error('Invalid pinyin table');
        }
        this.pinyinTable = [];
        while (reader.hasNext()) {
            const index = reader.nextInt16LE();
            const len = reader.nextInt16LE();
            this.pinyinTable[index] = reader.nextString(len);
        }
        return this.pinyinTable;
    }
    parseWords() {
        if (!this.pinyinTable) {
            this.parsePinyinTable();
        }
        const reader = new BufferReader(this.buf.slice(0x2628));
        const words = [];
        while (reader.hasNext()) {
            const homophoneNum = reader.nextInt16LE();
            const pinyinLen = reader.nextInt16LE();
            const pinyin = lodash_1.times(pinyinLen / 2).map(() => {
                if (!reader.hasNext()) {
                    return '?';
                }
                return this.pinyinTable[reader.nextInt16LE()];
            });
            lodash_1.times(homophoneNum).forEach(() => {
                if (!reader.hasNext()) {
                    return;
                }
                const wordBytes = reader.nextInt16LE();
                const word = reader.nextString(wordBytes);
                const extBytes = reader.nextInt16LE();
                const ext = reader.nextBuffer(extBytes);
                const frequency = ext.readInt16LE(0);
                words.push({ word, pinyin, frequency });
            });
        }
        return words;
    }
}
exports.default = ScelParser;
//# sourceMappingURL=index.js.map